@@ -47,6 +47,9 @@
 
 #include "vddk-structs.h"
 
+/* Enable extra disk info debugging with: -D vddk.diskinfo=1 */
+int vddk_debug_diskinfo;
+
 /* The VDDK APIs that we call.  These globals are initialized when the
  * plugin is loaded (by vddk_load).
  */
@@ -329,9 +332,22 @@ vddk_dump_plugin (void)
   printf ("vddk_default_libdir=%s\n", VDDK_LIBDIR);
   printf ("vddk_has_nfchostport=1\n");
 
-  /* XXX We really need to print the version of the dynamically
-   * linked library here, but VDDK does not provide it.
+#if defined(HAVE_DLADDR)
+  /* It would be nice to print the version of VDDK from the shared
+   * library, but VDDK does not provide it.  Instead we can get the
+   * path to the library using the glibc extension dladdr, and then
+   * resolve symlinks using realpath.  The final pathname should
+   * contain the version number.
    */
+  Dl_info info;
+  char *p;
+  if (dladdr (VixDiskLib_InitEx, &info) != 0 &&
+      info.dli_fname != NULL &&
+      (p = nbdkit_realpath (info.dli_fname)) != NULL) {
+    printf ("vddk_dll=%s\n", p);
+    free (p);
+  }
+#endif
 }
 
 /* XXX To really do threading correctly in accordance with the VDDK
@@ -458,6 +474,26 @@ vddk_get_size (void *handle)
 
   size = info->capacity * (uint64_t)VIXDISKLIB_SECTOR_SIZE;
 
+  if (vddk_debug_diskinfo) {
+    nbdkit_debug ("disk info: capacity: %" PRIu64 " (size: %" PRIi64 ")",
+                  info->capacity, size);
+    nbdkit_debug ("disk info: biosGeo: C:%" PRIu32 " H:%" PRIu32 " S:%" PRIu32,
+                  info->biosGeo.cylinders,
+                  info->biosGeo.heads,
+                  info->biosGeo.sectors);
+    nbdkit_debug ("disk info: physGeo: C:%" PRIu32 " H:%" PRIu32 " S:%" PRIu32,
+                  info->physGeo.cylinders,
+                  info->physGeo.heads,
+                  info->physGeo.sectors);
+    nbdkit_debug ("disk info: adapter type: %d",
+                  (int) info->adapterType);
+    nbdkit_debug ("disk info: num links: %d", info->numLinks);
+    nbdkit_debug ("disk info: parent filename hint: %s",
+                  info->parentFileNameHint ? : "NULL");
+    nbdkit_debug ("disk info: uuid: %s",
+                  info->uuid ? : "NULL");
+  }
+
   DEBUG_CALL ("VixDiskLib_FreeInfo", "info");
   VixDiskLib_FreeInfo (info);
 
@@ -487,7 +523,8 @@ vddk_pread (void *handle, void *buf, uint32_t count, uint64_t offset)
   count /= VIXDISKLIB_SECTOR_SIZE;
 
   DEBUG_CALL ("VixDiskLib_Read",
-              "handle, %" PRIu64 ", %" PRIu32 ", buffer", offset, count);
+              "handle, %" PRIu64 " sectors, %" PRIu32 " sectors, buffer",
+              offset, count);
   err = VixDiskLib_Read (h->handle, offset, count, buf);
   if (err != VIX_OK) {
     VDDK_ERROR (err, "VixDiskLib_Read");
@@ -520,7 +557,8 @@ vddk_pwrite (void *handle, const void *buf, uint32_t count, uint64_t offset)
   count /= VIXDISKLIB_SECTOR_SIZE;
 
   DEBUG_CALL ("VixDiskLib_Write",
-              "handle, %" PRIu64 ", %" PRIu32 ", buffer", offset, count);
+              "handle, %" PRIu64 " sectors, %" PRIu32 " sectors, buffer",
+              offset, count);
   err = VixDiskLib_Write (h->handle, offset, count, buf);
   if (err != VIX_OK) {
     VDDK_ERROR (err, "VixDiskLib_Write");
