@@ -49,12 +49,14 @@ main (int argc, char *argv[])
 {
   guestfs_h *g;
   int r;
+  const char *s;
   char *data;
 
   /* These languages currently fail completely when run under
    * valgrind, so skip them.
    */
-  if (getenv ("NBDKIT_VALGRIND") != NULL &&
+  s = getenv ("NBDKIT_VALGRIND");
+  if (s && strcmp (s, "1") == 0 &&
       (strcmp (LANG, "python") == 0 ||
        strcmp (LANG, "ruby") == 0 ||
        strcmp (LANG, "tcl") == 0)) {
@@ -109,6 +111,7 @@ main (int argc, char *argv[])
              program_name, filename, data, content);
     exit (EXIT_FAILURE);
   }
+  free (data);
 
   /* Run sync to test flush path. */
   if (guestfs_sync (g) == -1)
@@ -127,8 +130,8 @@ main (int argc, char *argv[])
   if (guestfs_umount (g, "/") == -1)
     exit (EXIT_FAILURE);
   const char *cmd[] = { "fallocate", "-nzl", "64k", "/dev/sda", NULL };
-  char *s = guestfs_debug (g, "sh", (char **) cmd);
-  free (s);
+  data = guestfs_debug (g, "sh", (char **) cmd);
+  free (data);
 
   if (guestfs_shutdown (g) == -1)
     exit (EXIT_FAILURE);
