@@ -31,9 +31,9 @@
 # OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 # SUCH DAMAGE.
 
+source ./functions.sh
 set -e
 set -x
-source ./functions.sh
 
 # Don't fail if certain commands aren't available.
 if ! ss --version; then
@@ -66,42 +66,15 @@ fi
 # Unfortunately qemu cannot do TLS over a Unix domain socket (nbdkit
 # probably can, although it is not tested).  Find an unused port to
 # listen on.
-for port in `seq 50000 65535`; do
+for port in {50000..65535}; do
     if ! ss -ltn | grep -sqE ":$port\b"; then break; fi
 done
 echo picked unused port $port
 
-nbdkit -P tls.pid -p $port -n --tls=require \
+cleanup_fn rm -f tls.pid tls.out
+start_nbdkit -P tls.pid -p $port -n --tls=require \
        --tls-certificates="$pkidir" example1
 
-# We may have to wait a short time for the pid file to appear.
-for i in `seq 1 10`; do
-    if test -f tls.pid; then
-        break
-    fi
-    sleep 1
-done
-if ! test -f tls.pid; then
-    echo "$0: PID file was not created"
-    exit 1
-fi
-
-pid="$(cat tls.pid)"
-
-# Kill the process on exit.
-cleanup ()
-{
-    status=$?
-    trap '' INT QUIT TERM EXIT ERR
-    echo $0: cleanup: exit code $status
-
-    kill $pid
-    rm -f tls.pid tls.out
-
-    exit $status
-}
-trap cleanup INT QUIT TERM EXIT ERR
-
 # Run qemu-img against the server.
 LANG=C \
 qemu-img info \
